# todo - 2 class doc

import logging
import os
from textwrap import dedent


def reformat_markdown_safe(text: str) -> str:
    # todo - 2 function doc

    text = text.replace("_", "\_")
    text = text.replace("*", "\*")
    return text


class TelegramFormatter(logging.Formatter):
    # todo 3 - class doc
    _TEMPLATE = dedent(
        """
    #tglogger
    *Logger Name:* {logger_name}
    *System Date:* {system_date} ({zone})
    *Level:* #{level_name}
    *Path and Line:* _{path}_:{lineno}
    *Function/Method:* _{func_name}_
    *Thread ID and Name:* \[{thread_id}] {thread_name}
    *Process ID and Name:* \[{process_id}] {process_name}

    *Message*
    {message}
    """
    ).strip()

    def format(self, record: logging.LogRecord):
        data = dict()

        data["logger_name"] = record.name

        if os.environ.get("DJANGO_SETTINGS_MODULE", None):
            from django.utils import timezone
            from django.conf import settings

            tz = getattr(settings, "TIME_ZONE", "No Timezone")
            use_tz = getattr(settings, "USE_TZ", False)

            if not use_tz:
                tz = "No Timezone"

            data["system_date"] = timezone.now()
            data["zone"] = tz
        else:
            from datetime import datetime

            data["system_date"] = datetime.now()
            data["zone"] = "No Timezone"

        data["level_name"] = record.levelname.lower()
        data["path"] = reformat_markdown_safe(record.pathname)
        data["lineno"] = record.lineno
        data["func_name"] = reformat_markdown_safe(record.funcName)
        data["thread_id"] = record.thread
        data["thread_name"] = record.threadName

        data["process_id"] = record.process
        data["process_name"] = record.processName
        data["message"] = record.getMessage()

        return self._TEMPLATE.format_map(data)



import logging
import os

from tglogger import request


class TelegramHandler(logging.Handler):
    def __init__(
        self,
        level=logging.ERROR,
        bot_token: str = os.environ["TELEGRAM_BOT_TOKEN"],
        receiver: str = os.environ["TELEGRAM_RECEIVER"],
    ):
        self.bot_token = bot_token
        self.receiver = receiver
        super().__init__(level)

    def emit(self, record):
        return request.send_log(self, record, self.receiver)



import logging
import tempfile
import traceback
import typing
import uuid

import requests

_BASE_URL = "https://api.telegram.org/bot{token}/{method}"


def _send_stack_trace(
    generic_info_response: requests.Response,
    handler: logging.Handler,
    record: logging.LogRecord,
    chat_id: int,
) -> typing.Union[None, requests.Response]:
    """
    Send stack trace for log in Telegram.
    """
    if record.exc_info is None or record.exc_info == (None, None, None):
        return None

    stack_trace_file = tempfile.NamedTemporaryFile(
        mode="w+b", prefix="stacktrace-{}".format(uuid.uuid1()), suffix=".txt"
    )

    tback = record.exc_info[1].__traceback__
    tback_data = traceback.format_tb(tback)

    for data in tback_data:
        stack_trace_file.write(data.encode())

    stack_trace_file.seek(0)

    reply_id = generic_info_response.json()["result"]["message_id"]
    response = requests.post(
        url=_BASE_URL.format(token=handler.bot_token, method="sendDocument"),
        data={
            "chat_id": chat_id,
            "caption": "Exception raised by **{}**.".format(
                record.exc_info[0].__name__
            ),
            "parse_mode": "markdown",
            "reply_to_message_id": reply_id,
        },
        files={"document": stack_trace_file},
    )

    stack_trace_file.close()
    return response


def send_log(
    handler: logging.Handler, record: logging.LogRecord, chat_id: int
) -> typing.Dict[str, requests.Response]:
    """
    Sends log to Telegram chat.
    """
    generic_info_response = requests.post(
        url=_BASE_URL.format(token=handler.bot_token, method="sendMessage"),
        data={
            "chat_id": chat_id,
            "text": handler.format(record),
            "parse_mode": "markdown",
        },
    )

    stack_trace_response = _send_stack_trace(
        generic_info_response, handler, record, chat_id
    )

    return {
        "generic_info_response": generic_info_response,
        "stack_trace_response": stack_trace_response,
        "django_settings_response": None,
    }



from setuptools import setup

import tglogger

with open("README.md", "r") as f:
    README = f.read()

with open("requirements.txt", "r") as f:
    DEPS = f.readlines()

with open("dev.requirements.txt", "r") as f:
    TEST_DEPS = f.readlines()

GITHUB_RELEASE_URL = "https://github.com/erayerdin/tglogger/archive/v{}.tar.gz"

setup(
    name="tglogger",
    version=tglogger.__version__,
    description="tglogger is a logger handler and formatter for Telegram bots.",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/erayerdin/tglogger",
    download_url=GITHUB_RELEASE_URL.format(tglogger.__version__),
    packages=("tglogger",),
    include_package_data=True,
    keywords="telegram messaging communication logging",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: Apache Software License",
        "Programming Language :: Python :: 3 :: Only",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Framework :: Django :: 1.11",
        "Framework :: Django :: 2.0",
        "Framework :: Django :: 2.1",
        "Framework :: Django :: 2.2",
        "Topic :: Communications :: Chat",
        "Topic :: System :: Logging",
        "Topic :: System :: Monitoring",
    ],
    author=tglogger.__author__,
    author_email="eraygezer.94@gmail.com",
    license="Apache License 2.0",
    tests_require=TEST_DEPS,
    install_requires=DEPS,
    zip_safe=False,
)
